<?php

/**
 * Plugin Name: Bosta WooCommerce
 * Description: WooCommerce integration for Bosta eCommerce
 * Author: Bosta
 * Author URI: https://www.bosta.co/
 * Version: 1.4
 * Requires at least: 5.0
 * Tested up to: 5.3.2
 * WC requires at least: 2.6
 * WC tested up to: 3.9.2
 * Text Domain: bosta-woocommerce
 * Domain Path: /languages
 *
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}
$citiesCode = array(
    "EG-01" => "Cairo",
    "EG-02" => "Alexandria",
    "EG-03" => "Sahel",
    "EG-04" => "Behira",
    "EG-05" => "Dakahlia",
    "EG-06" => "El Kalioubia",
    "EG-07" => "Gharbia",
    "EG-08" => "Kafr Alsheikh",
    "EG-09" => "Monufia",
    "EG-10" => "Sharqia",
    "EG-11" => "Isamilia",
    "EG-12" => "Suez",
    "EG-13" => "Port Said",
    "EG-14" => "Damietta",
    "EG-15" => "Fayoum",
    "EG-16" => "Bani Suif",
    "EG-17" => "Assuit",
    "EG-18" => "Sohag",
    "EG-19" => "Menya",
    "EG-20" => "Qena",
    "EG-21" => "Aswan",
    "EG-22" => "Luxor",
);
// add notice to config plugin
add_action('admin_notices', 'bosta_woocommerce_notice');
function bosta_woocommerce_notice()
{
    //check if woocommerce installed and activated
    if (!class_exists('WooCommerce')) {
        echo
        '<div class="error notice-warning text-bold">
        <p>
        <img src="' . esc_url(plugins_url('assets/images/bosta.svg', __FILE__)) . '" alt="Bosta" style="height:13px; width:25px;">
        <strong>' . sprintf(esc_html__('Bosta requires WooCommerce to be installed and active. You can download %s here.'), '<a href="https://woocommerce.com/" target="_blank">WooCommerce</a>') . '</strong>
        </p>
        </div>';
        return;
    }
}

add_action('admin_menu', 'bosta_setup_menu');
function bosta_setup_menu()
{
    //check if woocommerce is activated
    if (!class_exists('WooCommerce')) {
        return;
    }

    add_menu_page(
        'Test Plugin Page',
        'Bosta',
        'manage_options',
        'bosta-woocommerce',
        'bosta_setting',
        esc_url(plugins_url('assets/images/bosta.svg', __FILE__))
    );

    // link to plugin settings
    add_submenu_page(
        'bosta-woocommerce',
        'Setting',
        'Setting',
        'manage_options',
        'bosta-woocommerce',
        'bosta_setting'
    );

    // link to woocommerce orders
    add_submenu_page(
        'bosta-woocommerce',
        'Send Orders',
        'Send Orders',
        'manage_options',
        'bosta-woocommerce-orders',
        'bosta_orders'
    );

    // create pickup request
    add_submenu_page(
        'bosta-woocommerce',
        'Pickup Requests',
        'Pickup Requests',
        'manage_options',
        'bosta-woocommerce-create-pickup',
        'create_pickup_form'
    );

    // link to bosta shipments
    add_submenu_page(
        'bosta-woocommerce',
        'Track Bosta Orders',
        'Track Bosta Orders',
        'manage_options',
        'bosta-woocommerce-shipments',
        'bosta_dashboard'
    );
}

function bosta_setting()
{
    $redirect_url = admin_url('admin.php?') . 'page=wc-settings&tab=shipping&section=bosta';
    wp_redirect($redirect_url);
}

function bosta_orders()
{
    $redirect_url = admin_url('edit.php?') . 'post_type=shop_order&paged=1';
    wp_redirect($redirect_url);
}

function bosta_dashboard()
{
    $redirect_url = 'https://business.bosta.co/shipments';
    wp_redirect($redirect_url);
}

add_action('load-edit.php', 'wco_load', 20);
function wco_load()
{
    $screen = get_current_screen();
    if (!isset($screen->post_type) || 'shop_order' != $screen->post_type) {
        return;
    }

    add_filter("manage_{$screen->id}_columns", 'wco_add_columns');
    add_action("manage_{$screen->post_type}_posts_custom_column", 'wco_column_cb_data', 10, 2);
}

function wco_add_columns($columns)
{
    $order_total = $columns['order_total'];
    unset($columns['order_total']);

    $columns["bosta_status"] = __("Bosta Status", "themeprefix");
    $columns["bosta_tracking_number"] = __("Bosta Tracking Number", "themeprefix");
    $columns['order_total'] = $order_total;

    return $columns;
}

function wco_column_cb_data($colname, $orderId)
{
    if ($colname == 'bosta_status') {
        $status = get_post_meta($orderId, 'bosta_status', true);
        if (!empty($status)) {
            echo $status;
        } else {
            echo "---";
        }
    }

    if ($colname == 'bosta_tracking_number') {
        $trackingNumber = get_post_meta($orderId, 'bosta_tracking_number', true);
        if (!empty($trackingNumber)) {
            echo $trackingNumber;
        } else {
            echo "---";
        }
    }
}

add_action('manage_posts_extra_tablenav', 'create_pickup_top_bar_button', 20, 1);
function create_pickup_top_bar_button($which)
{
    global $pagenow, $typenow;

    if ('shop_order' === $typenow && 'edit.php' === $pagenow && 'top' === $which) {
        ?>
                <br>
                <br>
                <br>
        <div class="alignleft actions custom">
            <button type="submit" name="create_pickup" style="height:32px;" class="button" value="yes"><?php
echo __('Create Pickup', 'woocommerce'); ?></button>
        </div>
        <?php
}
    if ('shop_order' === $typenow && 'edit.php' === $pagenow &&
        isset($_GET['create_pickup']) && $_GET['create_pickup'] === 'yes') {
        $redirect_url = admin_url('admin.php?') . 'page=bosta-woocommerce-create-pickup';
        wp_redirect($redirect_url);
    }
}

function create_pickup_form()
{
    echo "
        <style>
        table {
        border-collapse: collapse;
        width: 95%;
        }

        th, td {
        padding: 8px;
        text-align:center;
        }

        form {
            text-align: center;
        }

        .data{
            width:300px;
            height:40px;
        }

        .error{
            text-align:center;
            color:red;
        }

        .updated{
            text-align:center;
            color:green;
        }

        </style>
        <form method='post' action='#'>
        <table>
        <thead>
        <tr>
            <th button-primary>Create Pickup Request</th>
        </tr>
        </thead>
            <tbody>

                <tr>
                   <td>
                    <label >Pickup Date</label>
                    <input require class='data' name='scheduledDate' type='date'/>
                   </td>
                </tr>
                <tr>
                   <td>
                        <label >Pickup Time</label>
                        <select require class='data' name='scheduledTimeSlot'>
                            <option value='10:00 to 13:00'>10:00AM to 01:00PM</option>
                            <option value='13:00 to 16:00'>01:00PM to 04:00PM</option>
                        </select>
                    </td>
                </tr>
            </tbody>
        </table>
        <input class='button button-primary' type='submit' name='create' value='Create'/>
        </form>";

    if (isset($_POST['create'])) {

        if (empty($_POST['scheduledDate'])) {
            echo "<div class='error'>Pickup Date is Required</div>";
        }

        if (empty($_POST['scheduledTimeSlot'])) {
            echo "<div class='error'>Pickup Time is Required<div>";
        }

        if (!empty($_POST['scheduledDate']) && !empty($_POST['scheduledTimeSlot'])) {
            create_pickup_action('create_pickup', $_POST['scheduledDate'], $_POST['scheduledTimeSlot']);
        }
    }
}

function create_pickup_action($action, $scheduledDate, $scheduledTimeSlot)
{
    if ($action != 'create_pickup') {
        return;
    }

    $APIKey = get_option('woocommerce_bosta_settings')['APIKey'];
    if (empty($APIKey)) {
        $redirect_url = admin_url('admin.php?') . 'page=wc-settings&tab=shipping&section=bosta';
        wp_redirect($redirect_url);
    }
    $pickupData = new stdClass();
    $pickupData->scheduledDate = $scheduledDate;
    $pickupData->scheduledTimeSlot = $scheduledTimeSlot;

    $result = wp_remote_post('https://app.bosta.co/api/v0/pickups',
        array('timeout' => 30,
            'method' => 'POST',
            'headers' => array(
                'Content-Type' => 'application/json',
                'authorization' => $APIKey,
            ),
            'body' => json_encode($pickupData),
        ));

    $result = json_decode($result['body']);
    if ($result->success === false) {
        echo "<div class='error'>" . $result->message . "<div>";
    } else {
        echo "<div class='updated'>Pickup Request Created Successfuly with id: " . $result->message->_id . "<div>";
    }

}

add_filter('bulk_actions-edit-shop_order', 'sync_bosta', 20, 1);
function sync_bosta($actions)
{
    $actions['sync_to_bosta'] = __('Send To Bosta', 'woocommerce');
    return $actions;
}

add_filter('handle_bulk_actions-edit-shop_order', 'sync_bosta_handle', 10, 3);
function sync_bosta_handle($redirect_to, $action, $order_ids)
{
    if ($action != 'sync_to_bosta') {
        return;
    }

    $APIKey = get_option('woocommerce_bosta_settings')['APIKey'];
    if (empty($APIKey)) {
        $redirect_url = admin_url('admin.php?') . 'page=wc-settings&tab=shipping&section=bosta';
        wp_redirect($redirect_url);
        return;
    }

    global $citiesCode;
    $args = array('post__in' => $order_ids);
    $allOrders = wc_get_orders($args);
    $formatedOrders = array();

    foreach ($allOrders as $order) {
        if (empty(get_post_meta($order->id, 'bosta_tracking_number', true))) {
            $items = $order->get_items();
            $desc = 'Products: ';
            foreach ($items as $item_id => $item_data) {
                $product = $item_data->get_product();
                $product_name = $product->get_name();
                $item_quantity = $item_data->get_quantity();
                $desc .= $product_name . '(' . $item_quantity . ') ';
            }

            $order = json_decode($order);
            $newOrder = new stdClass();
            $newOrder->id = $order->id;

            $newOrder->type = 10;
            // $newOrder->specs = new stdClass();
            // $newOrder->specs->packageDetails = new stdClass();
            // $newOrder->specs->packageDetails->description = $desc;
            $newOrder->notes = $desc . ' ' . $order->customer_note;
            $newOrder->businessReference = 'Woocommerce_' . $order->order_key;
            $newOrder->receiver = new stdClass();
            $newOrder->receiver->firstName = $order->shipping->first_name;
            $newOrder->receiver->lastName = $order->shipping->last_name;
            $newOrder->receiver->phone = $order->billing->phone;
            $newOrder->receiver->email = $order->billing->email;
            $newOrder->dropOffAddress = new stdClass();
            $newOrder->dropOffAddress->firstLine = $order->shipping->address_1;

            $address = explode('/', $order->shipping->state);
            $newOrder->dropOffAddress->city = array_search(ucwords(strtolower(trim($address[0]))), $citiesCode);
            $newOrder->dropOffAddress->zone = ucwords(strtolower(trim($address[1])));

            if ($order->payment_method == 'cod') {
                $newOrder->cod = (float) $order->total;
            }
            array_push($formatedOrders, $newOrder);
        }
    }

    for ($i = 0; $i < count($formatedOrders); $i++) {
        $id = $formatedOrders[$i]->id;
        unset($formatedOrders[$i]->id);

        $result = wp_remote_post('https://app.bosta.co/api/v0/deliveries',
            array('timeout' => 30,
                'method' => 'POST',
                'headers' => array(
                    'Content-Type' => 'application/json',
                    'authorization' => $APIKey,
                ),
                'body' => json_encode($formatedOrders[$i])));

        if ($result['response']['code'] != 201) {
            $result = json_decode($result['body']);
            if (gettype($result) == 'array') {
                $error = $result[0]->message;
            } else {
                $error = $result->message;
            }
            echo "Something went wrong: " . $error;
        } else {
            $result = json_decode($result['body']);
            if ($result->_id && empty(get_post_meta($id, 'bosta_delivery_id', true))) {
                add_post_meta($id, 'bosta_delivery_id', $result->_id);
            }

            if ($result->state->value && empty(get_post_meta($id, 'bosta_status', true))) {
                add_post_meta($id, 'bosta_status', $result->state->value);
            }

            if ($result->trackingNumber && empty(get_post_meta($id, 'bosta_tracking_number', true))) {
                add_post_meta($id, 'bosta_tracking_number', $result->trackingNumber);
            }

        }
    }
    $redirect_url = admin_url('edit.php?') . 'post_type=shop_order&paged=1';
    wp_redirect($redirect_url);
}

add_action('manage_posts_extra_tablenav', 'send_all_top_bar_button', 20, 1);
function send_all_top_bar_button($which)
{
    global $pagenow, $typenow;

    if ('shop_order' === $typenow && 'edit.php' === $pagenow && 'top' === $which) {
        ?>
        <div class="alignleft actions custom">
            <button type="submit" name="send_all_orders" style="height:32px;" class="button" value="yes">
        <?php
echo __('Send all orders to bosta', 'woocommerce');
        ?>
            </button>
        </div>
        <script type="text/JavaScript">
          document.getElementsByClassName("alignleft")[1].setAttribute("class", "alignright");
        </script>
        <?php
}
    if ('shop_order' === $typenow && 'edit.php' === $pagenow &&
        isset($_GET['send_all_orders']) && $_GET['send_all_orders'] === 'yes') {
        echo $_GET['send_all_orders'];
        $orders = wc_get_orders(array('return' => 'ids'));
        sync_bosta_handle('', 'sync_to_bosta', $orders);
    }

}

add_action('manage_posts_extra_tablenav', 'fetch_status_top_bar_button', 20, 1);
function fetch_status_top_bar_button($which)
{
    global $pagenow, $typenow;

    if ('shop_order' === $typenow && 'edit.php' === $pagenow && 'top' === $which) {
        ?>
        <div class="alignright actions custom">
            <button type="submit" name="fetch_status" style="height:32px;" class="button" value="yes"> <?php
echo __('Refresh status ', 'woocommerce') . '<img src=' . esc_url(plugins_url('assets/images/refresh.png', __FILE__)) . ' alt="Bosta" style="height:17px; width:20px;">'; ?></button>
        </div>
        <br>
        <br>
        <?php
}
    if ('shop_order' === $typenow && 'edit.php' === $pagenow &&
        isset($_GET['fetch_status']) && $_GET['fetch_status'] === 'yes') {
        echo $_GET['fetch_status'];
        $orders = wc_get_orders(array('return' => 'ids'));
        fetch_latest_status_action('fetch_latest_status', $orders);
    }
}

function fetch_latest_status_action($action, $order_ids)
{
    if ($action != 'fetch_latest_status') {
        return;
    }

    $orderArray = array();
    $trackingNumbers;
    $APIKey = get_option('woocommerce_bosta_settings')['APIKey'];
    if (empty($APIKey)) {
        $redirect_url = admin_url('admin.php?') . 'page=wc-settings&tab=shipping&section=bosta';
        wp_redirect($redirect_url);
        return;
    }

    foreach ($order_ids as $order) {
        $trackingNumber = get_post_meta($order, 'bosta_tracking_number', true);
        if (!empty($trackingNumber)) {

            $orderObject = new stdClass();
            $orderObject->id = $order;
            $orderObject->trackingNumber = $trackingNumber;
            array_push($orderArray, $orderObject);

            $trackingNumbers .= ($trackingNumbers ? ',' : '') . $trackingNumber;
        }
    }
    $url = 'https://app.bosta.co/api/v0/deliveries/search?trackingNumbers=' . $trackingNumbers;
    $result = wp_remote_get($url,
        array('timeout' => 30,
            'method' => 'GET',
            'headers' => array(
                'Content-Type' => 'application/json',
                'authorization' => $APIKey,
            ),
        ));

    if ($result['response']['code'] != 200) {
        var_dump($result);
        $result = json_decode($result['body']);
        if (gettype($result) == 'array') {
            $error = $result[0]->message;
        } else {
            $error = $result->message;
        }
        echo "Something went wrong: " . $error;
    } else {
        $result = json_decode($result['body']);

        $deliveries = $result->deliveries;
        for ($i = 0; $i < count($deliveries); $i++) {
            for ($j = 0; $j < count($orderArray); $j++) {
                if ($deliveries[$i]->trackingNumber == $orderArray[$j]->trackingNumber) {
                    update_post_meta($orderArray[$j]->id, 'bosta_status', $deliveries[$i]->state->value);
                }

            }
        }
        $redirect_url = admin_url('edit.php?') . 'post_type=shop_order&paged=1';
        wp_redirect($redirect_url);
    }
}

add_filter('bulk_actions-edit-shop_order', 'print_awb', 20, 1);
function print_awb($actions)
{
    $actions['print_bosta_awb'] = __('Print Bosta AirWaybill', 'woocommerce');
    return $actions;
}

add_filter('handle_bulk_actions-edit-shop_order', 'print_awb_handle', 10, 3);
function print_awb_handle($redirect_to, $action, $order_ids)
{
    if ($action != 'print_bosta_awb') {
        return;
    }

    $APIKey = get_option('woocommerce_bosta_settings')['APIKey'];
    if (empty($APIKey)) {
        $redirect_url = admin_url('admin.php?') . 'page=wc-settings&tab=shipping&section=bosta';
        wp_redirect($redirect_url);
        return;
    }

    $ids = '';
    foreach ($order_ids as $order) {
        $deliveryId = get_post_meta($order, 'bosta_delivery_id', true);
        if (!empty($deliveryId)) {
            $ids .= ($ids ? ',' : '') . $deliveryId;
        }
    }

    $url = 'https://app.bosta.co/api/v0/deliveries/awb?ids=' . $ids;
    $result = wp_remote_get($url,
        array('timeout' => 30,
            'method' => 'GET',
            'headers' => array(
                'Content-Type' => 'application/json',
                'authorization' => $APIKey,
            ),
        ));
    if ($result['response']['code'] != 200) {
        var_dump($result);
        $result = json_decode($result['body']);
        if (gettype($result) == 'array') {
            $error = $result[0]->message;
        } else {
            $error = $result->message;
        }
        echo "Something went wrong: " . $error;
    } else {
        $result = json_decode($result['body']);

        $decoded = base64_decode($result->data, true);

        header('Content-Type: application/pdf');
        header('Cache-Control: public, must-revalidate, max-age=0'); // HTTP/1.1
        header('Pragma: public');
        ob_clean();
        flush();
        echo $decoded;
    }
}

function action_woocommerce_update_order($order_get_id)
{
    $bostaStatus = get_post_meta($order_get_id, 'bosta_status', true);
    if ($bostaStatus != 'pending' && $bostaStatus != 'Created') {
        return;
    }

    global $citiesCode;
    $APIKey = get_option('woocommerce_bosta_settings')['APIKey'];
    $deliveryId = get_post_meta($order_get_id, 'bosta_delivery_id', true);
    $order = wc_get_order($order_get_id);

    $order = json_decode($order);
    $newOrder = new stdClass();
    $newOrder->notes = $order->customer_note;
    $newOrder->receiver = new stdClass();
    $newOrder->receiver->firstName = $order->shipping->first_name;
    $newOrder->receiver->lastName = $order->shipping->last_name;
    $newOrder->receiver->phone = $order->billing->phone;
    $newOrder->receiver->email = $order->billing->email;
    $newOrder->dropOffAddress = new stdClass();
    $newOrder->dropOffAddress->firstLine = $order->shipping->address_1;

    $address = explode('/', $order->shipping->state);
    $newOrder->dropOffAddress->city = array_search(ucwords(strtolower(trim($address[0]))), $citiesCode);
    $newOrder->dropOffAddress->zone = ucwords(strtolower(trim($address[1])));

    if ($order->payment_method == 'cod') {
        $newOrder->cod = (float) $order->total;
    }

    $result = wp_remote_request('https://app.bosta.co/api/v0/deliveries/' . $deliveryId,
        array('timeout' => 30,
            'method' => 'PUT',
            'headers' => array(
                'Content-Type' => 'application/json',
                'authorization' => $APIKey,
            ),
            'body' => json_encode($newOrder)));
};

add_action('woocommerce_update_order', 'action_woocommerce_update_order', 10, 1);
function wporg_add_custom_box()
{
    add_meta_box(
        'wporg_box_id',
        '',
        'wporg_custom_box_html',
        'shop_order'
    );

}

add_action('add_meta_boxes', 'wporg_add_custom_box');
function wporg_custom_box_html($post)
{
    $screen = get_current_screen();
    if (!isset($screen->post_type) || 'shop_order' != $screen->post_type) {
        return;
    }

    $APIKey = get_option('woocommerce_bosta_settings')['APIKey'];
    $trackingNumber = get_post_meta($post->ID, 'bosta_tracking_number', true);
    if (empty($trackingNumber)) {
        return;
    }

    $url = 'https://app.bosta.co/api/v0/deliveries/search?trackingNumbers=' . $trackingNumber;
    $result = wp_remote_get($url,
        array('timeout' => 30,
            'method' => 'GET',
            'headers' => array(
                'Content-Type' => 'application/json',
                'authorization' => $APIKey,
            ),
        ));

    if (is_wp_error($result)) {
        $error_message = $result->get_error_message();
        echo "Something went wrong: $error_message";
    } else {
        $result = json_decode($result['body']);
        $delivery = $result->deliveries[0];
    }
    if ($delivery->state->value != 'Created') {
        ?>
<script>
var div = document.createElement("div");
var p = document.createElement("p");
var textnode = document.createTextNode(" The order is being shipped by bosta. Any updating or deleting on the order info will not reflect to bosta system. For support email help@bosta.co");
p.appendChild(textnode);
div.appendChild(p);
div.setAttribute('class', 'error');
const parent = document.getElementsByClassName("wrap")[0];
parent.insertBefore(div, parent.children[3]);

 </script>
    <?php
}
}

add_action('wp_trash_post', 'custom_delete_function');
function custom_delete_function($id)
{
    $screen = get_current_screen();
    if (!isset($screen->post_type) || 'shop_order' != $screen->post_type) {
        return;
    }

    $bostaStatus = get_post_meta($id, 'bosta_status', true);
    if ($bostaStatus != 'pending' && $bostaStatus != 'Created') {
        return;
    }

    $APIKey = get_option('woocommerce_bosta_settings')['APIKey'];
    $deliveryId = get_post_meta($id, 'bosta_delivery_id', true);

    $result = wp_remote_request('https://app.bosta.co/api/v0/deliveries/' . $deliveryId,
        array('timeout' => 30,
            'method' => 'DELETE',
            'headers' => array(
                'Content-Type' => 'application/json',
                'authorization' => $APIKey,
            ),
        ));
}

add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'plugin_action_links');
function plugin_action_links($links)
{
    $plugin_links = array(
        '<a href="' . menu_page_url('bosta-woocommerce', false) . '">' . __('Settings') . '</a>',
    );

    return array_merge($plugin_links, $links);
}

add_action('plugins_loaded', 'init_bosta_shipping_class');
function init_bosta_shipping_class()
{
    //check if woocommerce is activated
    if (!class_exists('WooCommerce')) {
        return;
    }

    if (!class_exists('bosta_Shipping_Method')) {
        class bosta_Shipping_Method extends WC_Shipping_Method
        {
            public function __construct()
            {
                $this->id = 'bosta';
                $this->method_title = __('Bosta Shipping', 'bosta');
                $this->method_description = __('Custom Shipping Method for bosta', 'bosta');

                $this->init();
                $this->enabled = isset($this->settings['enabled']) ? $this->settings['enabled'] : 'yes';
                $this->title = isset($this->settings['title']) ? $this->settings['title'] : __('bosta Shipping', 'bosta');

            }

            function init()
            {
                $this->init_form_fields();
                $this->init_settings();
                add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
            }

            function init_form_fields()
            {
                $this->form_fields = array(
                    'APIKey' => array(
                        'title' => __('APIKey', 'bosta'),
                        'type' => 'text',
                    ),
                );
            }
        }
    }
}

add_action('woocommerce_shipping_init', 'init_bosta_shipping_class');
function add_bosta_shipping_method($methods)
{
    $methods[] = 'bosta_Shipping_Method';
    return $methods;
}

add_filter('woocommerce_shipping_methods', 'add_bosta_shipping_method');

add_filter('woocommerce_states', 'bosta_cities_and_zones');
function bosta_cities_and_zones($states)
{
    $states['EG'] = array(
        'Cairo / Cairo' => 'Cairo / Cairo',
        'Cairo / Giza & Haram' => 'Cairo / Giza & Haram',
        'Cairo / Maadi & Muqattam' => 'Cairo / Maadi & Muqattam',
        'Cairo / New Cairo' => 'Cairo / New Cairo',
        'Cairo / New Cities' => 'Cairo / New Cities',
        'Cairo / October' => 'Cairo / October',

        'Alexandria / Agmey & Amreya' => 'Alexandria / Agmey & Amreya',
        'Alexandria / Downtown Alex' => 'Alexandria / Downtown Alex',
        'Alexandria / East Alex' => 'Alexandria / East Alex',
        'Alexandria / El Gomrok' => 'Alexandria / El Gomrok',
        'Alexandria / El Montaza' => 'Alexandria / El Montaza',

        'Sahel / Sahel' => 'Sahel / Sahel',

        'Behira / Behira' => 'Behira / Behira',

        'Dakahlia / Dakahlia' => 'Dakahlia / Dakahlia',

        'El Kalioubia / El Kalioubia' => 'El Kalioubia / El Kalioubia',

        'Gharbia / Gharbia' => 'Gharbia / Gharbia',

        'Kafr Alsheikh / Kafr Alsheikh' => 'Kafr Alsheikh / Kafr Alsheikh',

        'Monufia / Monufia' => 'Monufia / Monufia',

        'Sharqia / Sharqia' => 'Sharqia / Sharqia',

        'Isamilia / Isamilia' => 'Isamilia / Isamilia',

        'Suez / Suez' => 'Suez / Suez',

        'Port Said / Port Said' => 'Port Said / Port Said',

        'Damietta / Damietta' => 'Damietta / Damietta',

        'Fayoum / Atsa' => 'Fayoum / Atsa',
        'Fayoum / Ebsheway' => 'Fayoum / Ebsheway',
        'Fayoum / Fayoum' => 'Fayoum / Fayoum',
        'Fayoum / Tamya' => 'Fayoum / Tamya',
        'Fayoum / Yousef Elsedeq' => 'Fayoum / Yousef Elsedeq',

        'Bani Suif / Bani Suif' => 'Bani Suif / Bani Suif',
        'Bani Suif / Biba' => 'Bani Suif / Biba',
        'Bani Suif / Elfashn' => 'Bani Suif / Elfashn',
        'Bani Suif / Elwasty' => 'Bani Suif / Elwasty',
        'Bani Suif / Ihnasya' => 'Bani Suif / Ihnasya',
        'Bani Suif / Nasser' => 'Bani Suif / Nasser',
        'Bani Suif / New Beni Suef' => 'Bani Suif / New Beni Suef',
        'Bani Suif / Samsta' => 'Bani Suif / Samsta',

        'Assuit / Abanoub' => 'Assuit / Abanoub',
        'Assuit / Abo Teg' => 'Assuit / Abo Teg',
        'Assuit / Asyut' => 'Assuit / Asyut',
        'Assuit / Dayrout' => 'Assuit / Dayrout',
        'Assuit / Elbadary' => 'Assuit / Elbadary',
        'Assuit / Elfath' => 'Assuit / Elfath',
        'Assuit / Elghanaym' => 'Assuit / Elghanaym',
        'Assuit / ElQawsaia' => 'Assuit / ElQawsaia',
        'Assuit / Manfalout' => 'Assuit / Manfalout',
        'Assuit / New Asyut' => 'Assuit / New Asyut',
        'Assuit / Sadfa' => 'Assuit / Sadfa',
        'Assuit / Sahel Selim' => 'Assuit / Sahel Selim',

        'Sohag / Dar Elsalam' => 'Sohag / Dar Elsalam',
        'Sohag / Elbalina' => 'Sohag / Elbalina',
        "Sohag / Elmansha'a" => "Sohag / Elmansha'a",
        'Sohag / Elosiryat' => 'Sohag / Elosiryat',
        'Sohag / Gerga' => 'Sohag / Gerga',
        'Sohag / Johannah' => 'Sohag / Johannah',
        'Sohag / Juhaynah' => 'Sohag / Juhaynah',
        'Sohag / Saqultah' => 'Sohag / Saqultah',
        'Sohag / Sohag' => 'Sohag / Sohag',
        'Sohag / Tahta' => 'Sohag / Tahta',
        'Sohag / Tama' => 'Sohag / Tahta',

        'Menya / Abo Kerkas' => 'Menya / Abo Kerkas',
        'Menya / Bani Mazar' => 'Menya / Bani Mazar',
        'Menya / Der Mwas' => 'Menya / Der Mwas',
        'Menya / Eladwa' => 'Menya / Eladwa',
        'Menya / Maghagha' => 'Menya / Maghagha',
        'Menya / Mallawi' => 'Menya / Mallawi',
        'Menya / Menya' => 'Menya / Menya',
        'Menya / Metay' => 'Menya / Metay',
        'Menya / New Minya' => 'Menya / New Minya',
        'Menya / Samalout' => 'Menya / Samalout',

        'Qena / Abo Tesht' => 'Qena / Abo Tesht',
        'Qena / Elwagf' => 'Qena / Elwagf',
        'Qena / Farshot' => 'Qena / Farshot',
        'Qena / Nagaa Hammadi' => 'Qena / Nagaa Hammadi',
        'Qena / Naqadah' => 'Qena / Naqadah',
        'Qena / Qeft' => 'Qena / Qeft',
        'Qena / Qena' => 'Qena / Qena',
        'Qena / Qos' => 'Qena / Qos',
        'Qena / Shana' => 'Qena / Shana',

        'Aswan / Abo Elresh' => 'Aswan / Abo Elresh',
        'Aswan / Abu Sinbil' => 'Aswan / Abu Sinbil',
        'Aswan / As Sibaiyyah' => 'Aswan / As Sibaiyyah',
        'Aswan / Aswan' => 'Aswan / Aswan',
        'Aswan / Draw' => 'Aswan / Draw',
        'Aswan / Edfu' => 'Aswan / Edfu',
        'Aswan / Elbosyliah' => 'Aswan / Elbosyliah',
        'Aswan / Elradesia' => 'Aswan / Elradesia',
        'Aswan / Kalabsha' => 'Aswan / Kalabsha',
        'Aswan / Kom Ombo' => 'Aswan / Kom Ombo',
        'Aswan / Nasr Elnouba' => 'Aswan / Nasr Elnouba',
        'Aswan / Sahary' => 'Aswan / Sahary',

        'Luxor / Arment' => 'Luxor / Arment',
        'Luxor / Elbyadia' => 'Luxor / Elbyadia',
        'Luxor / Elqarna' => 'Luxor / Elqarna',
        'Luxor / Eltod' => 'Luxor / Eltod',
        'Luxor / Elzenia' => 'Luxor / Elzenia',
        'Luxor / Isna' => 'Luxor / Isna',
        'Luxor / Luxor' => 'Luxor / Luxor',

    );
    return $states;

}

add_filter('woocommerce_checkout_fields', 'custom_override_checkout_fields');
function custom_override_checkout_fields($fields)
{
    unset($fields['billing']['billing_city']);
    unset($fields['shipping']['shipping_city']);
    return $fields;
}

add_filter('woocommerce_default_address_fields', 'rename_state_label', 9999);
function rename_state_label($fields)
{
    $fields['state']['label'] = 'City / Zone';
    return $fields;
}